;+
; NAME:
; 	Cont_Surf
; PURPOSE:
;	User interface to facilitate the use of the three 
;	dimensional standard functions
;	"Contour" and "Surface".  It is possible to change  
;	contour parameters: annotation, character sizes, 
;	colors used to represent the levels, labels of the levels, 
;	line styles and thickness,  maximum value and number of 
;	levels drawn. An option for smoothing is given and 
;	surface or contour plots can be printed and written in 
;	PostScript files.
; CATEGORY:
;	Quasi - Image Processing.
; CALLING SEQUENCE:
; 	Cont_Surf, image [, xAxis,  yAxis]
; INPUTS:
;	image: The  two-dimensionnal array to be contoured
;		or surfaced. If "image" contains more than
;		200 elements in one direction, it is
;		compressed to that number of elements,
;		using bilinear interpolation. "image"
;		can be of any type.
;	xAxis, yAxis: These two 1D array parameters (any type) 
;		of dimensions
;		corresponding to the array "image" contain 
;		axis values if they are not ranging  from 0 to
;		the number of elements minus 1.
; KEYWORDS:
;	FILENAME: The name of the PostScript file. 
;		If not present, the file will have
;		the same name as the title of the 
;		(surface or contour) plot.
;	LANDSCAPE: If present, the PostScript file is written in
;		landscape mode, otherwise in portrait mode.
; SIDE EFFECTS:
;	As all user interfaces, it opens windows on
;	X-windows terminals,   or erases the display on vt-type
;	terminals. Files can be produced, which will be written 
;	in the current directory.
;-
; MODIFICATION HISTORY
;	Created: A.Csillaghy May 91
;		Institute of Astronomy, ETH Zurich

PRO Change_Pars,annotation,charsize, colors,labels, linestyle, $
	thick, maxVal, nLevels

mainMenuLength = 11
mainMenu = StrArr(mainMenuLength)


  mainMenu(0) = 'Annotation: '+annotation
  mainMenu(1) = 'Characters Size: '+ STRING(charsize)
  mainMenu(2) = 'Colors Used: '+STRING(colors)
  mainMenu(3) = 'Labels Used: '+labels
  mainMenu(4) = 'Lines Style: '+ STRING(linestyle) 
  mainMenu(5) = 'Thickness Of Lines: ' + STRING(thick)
  mainMenu(7) = 'Maximum Value: ' + STRING(maxVal)
  mainMenu(8) = 'Number of Levels: ' + STRING( nLevels )
  mainMenu(mainMenuLength-2) = 'Return to Preceeding Menu'
  mainMenu(mainMenuLength-1) = 'Redisplay This Menu'

REPEAT BEGIN
    
  Clear_Screen
  choice = WMenu(['Contour Parameters: Choose what to Change',$
	mainMenu ], TITLE = 0 )

  CASE choice OF
    1: BEGIN
          temp = ''
          annotation = ''
          REPEAT BEGIN
            Read,'New annotation (<return> alone to stop): ',temp
            annotation = [annotation, temp]
          END UNTIL temp EQ ''
          IF N_Elements( annotation ) GT 2 THEN BEGIN
            annotation = annotation( 1: N_Elements(annotation)-2 )
            mainMenu(0) = 'Annotations: '
            FOR i=0, N_Elements( annotation )-1 DO $
              mainMenu(0) = mainMenu(0) + annotation(i) + ' '
          ENDIF
        END
    2: BEGIN
         Print,'A normal character size is 1.0'
         Read,'New character size: ',charsize
         mainMenu(1) = 'Characters Size: '+ STRING(charsize)
       END 
    3: BEGIN
         Print,'Colors are ranging from 0 to 255 '
         Read,'Number of colors: ',nColors
         colors = BytArr(nColors)
         Read,'New colors (separated by commas): ',colors
         mainMenu(2) = 'Colors Used: '+STRING(colors)
        END
    4: BEGIN
          temp = ''
          REPEAT BEGIN
            Read,'New label (separated by <return>): ',temp
            label = [label, temp]
          END UNTIL temp EQ ''
         mainMenu(2) = 'Labels Used: '+STRING(labels)
        END
    5: BEGIN
         Print,'Line Styles: '
         Print,'                    0: Solid'
         Print,'                    1: Dotted'
         Print,'                    2: Dashed'
         Print,'                    3: Dash Dot'
         Print,'                    4: Dash Dot Dot Dot'
         Print,'                    5: Long Dashes'
         Read,'Number of line styles: ', nls
         lineStyle = BytArr(nls)
         Read,'New line styles (separated by commas): ',linestyle
         mainMenu(4) = 'Lines Style: '+ STRING(linestyle) 
        END
    6: BEGIN
          Read,'New tickness of lines: ',thick
          mainMenu(5) = 'Thickness Of Lines: ' + STRING(thick)
        END
    8: BEGIN
         Read,'New maximum value: ',maxVal
         mainMenu(7) = 'Maximum Value : ' + STRING(maxVal)
        END
    9: BEGIN
         Read,'New number of levels: ',nLevels
         mainMenu(8) = 'Number of Levels: ' + STRING( nLevels )
         END
  ELSE:
  END
  
END UNTIL choice EQ mainMenuLength-1

END

PRO Test_Axis, axis, nels, STATUS = status

  status = "Done"

  CASE N_Elements(axis) OF
    nels:
    0: axis = IndGen(nels)
    ELSE: BEGIN
      Print,' The axis dimension should be equal to the'
      Print,' corresponding array dimension '
      Print, ''
      status = 'Not Done'
    END
  ENDCASE

END ; Test Axes

PRO Cont_Surf, imageP, xP,yP, FILENAME = fileName, $
	LANDSCAPE = landscape, $
	WINNR = winNr, SIZES_ON = sizes_on


IF N_Params() EQ 0 THEN BEGIN
  Error,  5, 'Cont_Surf'
  RETURN
ENDIF

oldP = !p & oldX = !x & oldY = !y

IF !d.name EQ 'X' THEN BEGIN
  IF Keyword_Set( WINNR ) THEN Window, winNR $
  ELSE Window, 0
ENDIF


pixelLimit = 200
nx = N_Elements( imageP(*,0))
ny = N_Elements( imageP(0,*))

Test_Axis, xP, nx, STATUS = status
IF status  NE "Done" THEN RETURN
Test_Axis, yP, ny, STATUS = status
IF status NE "Done" THEN RETURN

CASE 1 OF
  (nx GT pixelLimit) AND (ny GT pixelLimit): BEGIN
      image = ConGrid( imageP, pixelLimit, pixelLimit, /INTERP )
      x = Congrid1D( xP, pixelLimit, 1, /INTERP )
      y = Congrid1D( yP, pixelLimit, 1, /INTERP )
    END
  (nx GT pixelLimit): BEGIN
      image = Congrid( imageP, pixelLimit, ny, /INTERP )
      x = Congrid1D( xP, pixelLimit, 1 ) & y = yP
    END
  (ny GT pixelLimit): BEGIN
      image = Congrid( imageP, nx, pixelLimit )
      y = Congrid1D( yP, pixelLimit,1 ) & y = yP
    END
  (nx LE pixelLimit) AND (ny LE pixelLimit): BEGIN
    image = imageP & x = xP & y = yP
  END
ENDCASE

mainMenuLength = 8
mainMenu = StrArr(mainMenuLength)

mainMenu(0) = 'Contour'
mainMenu(1) = 'Surface'
mainMenu(2) = 'Smooth Data ... '
mainMenu(3) = 'Change Contour Parameters ... '
mainMenu(4) = 'Make Contour PostScript File / Print ...'
mainMenu(5) = 'Make Surface PostScript File / Print ...' 
mainMenu(6) = 'Exit this Menu'
mainMenu(7) = 'Redisplay this Menu'

smoothP = 0
annotation = [''] & charsize = 1 & colors = [255] & labels = ['']
linestyle = [0] & thick = [1] & nLevels = 5 & maxVal = MAX(image)

Set_XY, Min(x), Max(x), Min(y), Max(y)
!p.font = 0

REPEAT BEGIN
  choice = General_Menu(mainMenu,$
	'Contour and Surface: Choose Option' )
  
  CASE choice OF
    1: IF smoothP LE 1 THEN $
	Contour, image, x,y, $
		C_ANNOTATION = annotation, $
		C_CHARSIZE = charsize, $
		C_COLORS = colors, $
		C_LABELS = labels,$
   		NLEVELS = nlevels, $
                 		MAX_VALUE = maxVal $
        ELSE Contour, Smooth(image, smoothP) , x,y , $
		C_ANNOTATION = annotation, $
		C_CHARSIZE = charsize, $
		C_COLORS = colors, $
		C_LABELS = labels, $
   		NLEVELS = nlevels, $
                 		MAX_VALUE = maxVal 
    2: IF smoothP LE 1 THEN Surface, BytScl(image), x, y, $
	TITLE = '' ,SUBTITLE = '' , XTITLE = '', $
	YTITLE = '' , XSTYLE = 4,  YSTYLE = 4, $
	ZTITLE = '', ZSTYLE = 4, /SKIRT $
        ELSE Surface, BytScl(Smooth(image, smoothP)), x, y, $
	TITLE = '' ,SUBTITLE = '' , XTITLE = '', $
	YTITLE = '' , XSTYLE = 4,  YSTYLE = 4, $
	ZTITLE = '', ZSTYLE = 4, /SKIRT
    3: BEGIN
          Clear_Screen
          Read_Test,' New smooth factor ', smoothP,  tmp, $
	MAXIMUM = nx < ny < 200., MINIMUM = 1.
          smoothP = Fix( tmp )
        END
    4: Change_Pars, annotation,charsize, colors,labels, linestyle, $
	thick, maxVal, nlevels
    5: BEGIN
          IF NOT Keyword_Set( FILENAME )THEN $
	filename = 'enrad.ps'
          IF NOT Keyword_Set(LANDSCAPE) THEN landsc = 0
          dev = !d.name
          Set_Plot,'PS'
          IF Keyword_Set( SIZES_ON ) THEN BEGIN
            GetImageSizes, xPos, yPos, width, height, /PS 
            GetMiscPs, encaps, landsc
            IF landsc THEN BEGIN
              Device, FILENAME = fileName,  /TIMES, /BOLD, $
	XOFFSET = xPos, $
	YOFFSET = yPos, XSIZE = width, YSIZE = height, $
	ENCAPSULATED = encaps
              Device, OUTPUT= STRING(1000*(21-2*xPos))+ $
	' 3000 translate 90 rotate'                 
            ENDIF ELSE BEGIN
              Device, /TIMES, $
	FILENAME = fileName, XOFFSET = xPos, $
	YOFFSET = yPos, XSIZE = width, YSIZE = height, $
	/PORTRAIT, ENCAPSULATED = encaps
            ENDELSE
          ENDIF ELSE $
            Device, /TIMES, $
	FILENAME = fileName, LANDSCAPE = landscape
          IF smoothP LE 1 THEN $
	Contour,image, x,y, $
		C_ANNOTATION = annotation, $
		C_CHARSIZE = charsize, $
		C_LABELS = labels, $
   		NLEVELS = nlevels, $
                 		MAX_VALUE = maxVal  $
        ELSE Contour, Smooth(image, smoothP) , x,y , $
		C_ANNOTATION = annotation, $
		C_CHARSIZE = charsize, $
		C_LABELS = labels, $
   		NLEVELS = nlevels, $
                 		MAX_VALUE = maxVal 
          Device, /CLOSE_FILE
          Set_Plot, dev
        END
    6: BEGIN
          dev = !d.name
          Set_Plot,'PS'
          Device, /TIMES, $
	FILENAME ='enrad.ps',$
	/LANDSCAPE
          IF smoothP LE 1 THEN Surface, BytScl(image), x,y, $
	TITLE = '' ,SUBTITLE = '' , XTITLE = '', $
	YTITLE = '' , XSTYLE = 4,  YSTYLE = 4, $
	ZTITLE = '', ZSTYLE = 4, /SKIRT  $
          ELSE Surface, BytScl(Smooth(image, smoothP)), x, y, $
	TITLE = '' ,SUBTITLE = '' , XTITLE = '', $
	YTITLE = '' , XSTYLE = 4,  YSTYLE = 4, $
	ZTITLE = '', ZSTYLE = 4, /SKIRT
          Device,/CLOSE_FILE
          Set_Plot, dev
        END
  ELSE:
  ENDCASE
  IF (choice EQ 1) OR (choice EQ 2) AND (!d.name NE 'X') THEN $
    BEGIN
    Print, 'Hit any key to continue'
    HAK
    Erase
  END
  IF choice EQ 5 OR choice EQ 6 THEN Spawn, 'lpr -Pps enrad.ps'
    


END UNTIL choice EQ mainMenuLength-1

!p = oldP & !x = oldX & !y = oldY

END
