;+
; NAME:
; 	Tv_Zoom
; PURPOSE:
;	Zoom interactively an image:
;	1. by defining the corners of the zoom window on the 
;		display with the mouse;
;	2. By defining the x or y boundaries on the window display
;		with the mouse
;	3. By entering the x- or y-boundaries on the terminal
;		window with the keyboard;
;	4. By defining the zoom to be in full resolution around
;		the cursor on the display window.
; CATEGORY:
;	Image Processing
; CALLING SEQUENCE:
;	Tv_Zoom, xAxis, yAxis, xMin, xMax, yMin, yMax
; INPUT ARGUMENTS:
;	image: the image to be zoomed
;	xMin, xMax, yMin, yMax: The pixel number
;		of the part of the image currently displayed.
;	xAxis, yAxis: must be given  for the static zooms, 1D
; OUTPUT ARGUMENTS:
;	xMin, xMax, yMin, yMax: The new pixel coordinates of
;		the zoom boundary.
; KEYWORDS:
;	WINNR: to be specified if the image is not displayed
;		in the current window.
;-
; MODIFICATION HISTORY:
;	Created, 4/91 A.Csillaghy,
;		Institute of Astronomy, ETH Zurich
;	Modified strongly in August 1991, A.Cs.


PRO ZoomStat, axis, minValPixel, maxValPixel, dir

  IF minValPixel GT maxValPixel THEN BEGIN
    tmp = minValPixel 
    minValPixel = maxValPixel
    maxValPixel = tmp
  ENDIF

  hmsDir = GetHMS()*(dir EQ 'X')
  n = N_Elements( axis )

  minimum = axis(minValPixel) < axis(maxValPixel)
  maximum = axis(maxValPixel) > axis( minValPixel)
  Read_Test,'Least value ', minimum, minVal, HMS =hmsDir
  Read_Test,'Greatest value ', maximum, maxVal, HMS = hmsDir

  IF axis( minValPixel ) GT axis( maxValPixel ) THEN BEGIN
    minValPixel = MAX(Where( axis GE minVal )) > 0
    maxValPixel = MIN(Where( axis LE maxVal )) < (n-1)
  ENDIF ELSE BEGIN
    minValPixel = MIN(Where( axis GE minVal )) > 0
    maxValPixel = MAX(Where( axis LE maxVal )) < (n-1)
  ENDELSE

  IF minValPixel GT maxValPixel THEN BEGIN
    tmp = minValPixel 
    minValPixel = maxValPixel
    maxValPixel = tmp
  ENDIF

END


PRO SetAround,  center, minimum, maximum, length, pos

   n = maximum - minimum
    IF n  GT length THEN BEGIN
      center = center - pos >  0 < length
      half = length/2	;image AND device pixel
      center = FIX(FLOAT(n*center)/FLOAT(length)) + minimum       
      IF center LT minimum+half THEN BEGIN
        maximum = minimum + length - 1
      ENDIF ELSE IF center GT maximum-half THEN BEGIN
        minimum = maximum - length +1
      ENDIF ELSE BEGIN
        minimum = (center - half) 
        maximum = (center + half -1) 
      ENDELSE
    ENDIF

END


PRO Tv_Zoom,xAxis, yAxis , xMin,xMax,yMin,yMax,$
	WINNR=winNr, STATUS = status

status = "Done"
xWin = !d.name EQ 'X'

IF N_Params() NE 6 THEN BEGIN
  Print,'Wrong number of parameters ...'
  status = "Not Done"
  RETURN
ENDIF
IF N_Elements(xMin) EQ 0 THEN BEGIN
  xMin = 0 & xMax = N_Elements(xAxis)-1
  yMin = 0 & yMax = N_Elements(yAxis)-1
END
  
IF NOT Keyword_Set(WINNR) THEN winNr=!d.window

IF xWin THEN BEGIN
  GetImageSizes, xPos, yPos, width, height
  GetOffsets, d1, imageOffset
  GetBreaks, dxBreaks, dyBreaks
  nxBreaks = N_Breaks( dxBreaks )
  nyBreaks = N_Breaks( dyBreaks )
ENDIF


Clear_Screen

zoomMenu = StrArr(8)

zoomMenu(0) = 'Choose the type of Zoom'
zoomMenu(1) = 'X and Y Zoom Interactive ... '
zoomMenu(2) = 'X Zoom Interactive ... '
zoomMenu(3) = 'Y Zoom Interactive ... '
zoomMenu(4) = 'X Zoom Static ... '
zoomMenu(5) = 'Y Zoom Static ... '
zoomMenu(6) = 'Full Resolution Around Cursor ... '
zoomMenu(7) = 'Return to Display Menu'

zoomType = General_Menu( zoomMenu(1:7), zoomMenu(0), $
		OPAQUE = [1,1,1,0,0,1, 0]  )

nx = N_Elements(xAxis(xMin:xMax))
ny = N_Elements(yAxis(yMin:yMax))

CASE 1 OF

  (zoomType LE 3): BEGIN

    title='Zoom a part of the Display'
    WShow,winNr
    winMsg=WMesg( $
      'Position the mouse cursor to a point and press a button',$
      title,100,800)
    WSet,winNr
    Cursor,x0,y0,/DEVICE, /UP
    WDelete,winMsg
    winMsg=WMesg($
      'Position now to the other point and press again a button',$
      title,100,800)
    WSet,winNr
    Cursor,x1,y1,/DEVICE, /UP
    WDelete,winMsg

    CASE zoomType OF
      1 : Plots, [ x0, x1, x1, x0, x0 ], [ y0, y0, y1, y1, y0 ], /DEVICE
      2 : BEGIN
             
              totalHeight = height + nyBreaks*imageOffset + yPos
              Plots, [ x0, x1, x1, x0, x0 ], [ yPos, yPos, totalHeight, $
	totalHeight, yPos ], /DEVICE
           END
      3 : BEGIN
              totalWidth = width + nxBreaks*imageOffset + xPos
              Plots,[xPos, totalWidth, totalWidth, xPos, xPos ], $
	[ y0, y0, y1, y1, y0 ], /DEVICE
            END
    ENDCASE

    IF (zoomType EQ 1) OR (zoomType EQ 2) THEN BEGIN
      x0 = DevToPix( x0, xPos, width, dxBreaks, imageOffset, nx )
      x1 = DevToPix( x1, xPos, width, dxBreaks, imageOffset, nx )
      xm = xMin
      xMin = x0 < x1 + xm
      xMax = x0 > x1 + xm 
    ENDIF
    IF (zoomType EQ 1) OR (zoomType EQ 3) THEN BEGIN  
      y0 = DevToPix( y0, yPos, height, dyBreaks, imageOffset, ny )
      y1 = DevToPix( y1, yPos, height, dyBreaks, imageOffset, ny )
      ym = yMin
      yMin = y0 < y1 + ym
      yMax = y0 > y1 + ym 
    ENDIF 
  END

  (zoomType EQ 4) :  ZoomStat, xAxis, xMin, xMax, 'X'

  (zoomType EQ 5) :  ZoomStat, yAxis, yMin, yMax,  'Y'

  (zoomType EQ 6) : BEGIN
     
    IF (yMax-yMin LE height) AND $
	(xMax-xMin LE width) THEN BEGIN
      Print,'The image is already in full resolution ... '
      Print, ' '
      RETURN
    ENDIF

    title='Full Resolution Around the Cursor'
    WShow,winNr
    winMsg=WMesg( $
      ['Position the cursor where you want the full resolution',$
       'and press a button'], $
      title,100,800)
    WSet,winNr
    Cursor,x0,y0,/DEVICE
    WDelete,winMsg

    SetAround, x0, xMin, xMax, width, xPos
    SetAround, y0, yMin, yMax, height, yPos
   
    END

  ELSE: status = "Not Done"
  ENDCASE

 
END
