PRO MDispRead, file, x, y, q, dx, dy, $
	LINESTYLE = lineStyle, THICK = thick, $
	COLOR = color, SUBTITLE = subTitle, $
	TITLE = title, XTITLE = xTitle, YTITLE = yTitle, $
	TEXT = text, USERSYM = userSym, STATUS = status, $
	PARLIST = parList, PARCOLLIST = parColList, $
	VARPLACE = varPlace

;+
; NAME:
;	MDisp Read
; PURPOSE:
;	Reads mdisp-files.
; CALLING SEQUENCE:
;	MDispRead, fileName, x, y [, qualifier, dx, dy]
; INPUT:
;	fileName: the complete name of the file (with the extension
;		".mdisp"
; OUTPUTS:
;	x, y : the points of the plot
;	qualifier: the qualifiers corresponding to the !psym 
;		variable value
;	dx, dy: the errors in x and y direction, for plotting of 
;		error bars. If the error bars are symmetric,
;		then the arrays are of the dimension of x and y,
;		else they have 2 dimensions with the second 
;		dimension = 2
; KEYWORDS:
;	LINESTYLE: correspond to the values of !p.linestyle
;		solid line: linestyle = 0
;	THICK: line thickness. Normal line: thick = 1
;	SUBTITLE, TITLE, XTITLE, YTITLE: The plot
;		 description
;	STATUS: "Done" if a file was read, "Not Done" else.
; MODIFICATION HISTORY:
;	Created in May 1991 by A.Csillaghy
;		Institute of Astronomy, ETH Zurich
;	Modified in November 1991, A.Cs.
;	Modified in Nov 92 for Wave 4 (DC_Read)
;	Adapted for IDL in March 96 (ACs)
;-

  status = "Not Done"

  On_IOError, exit

  varList = ['x','y','q', 'dyUp', 'dxLeft', 'dyDown', 'dxRight']

  IF NOT Keyword_Set( PARLIST ) THEN BEGIN

    Print,' MDisp file read '

    OpenR, unit, file, /GET_LUN

; initializations:


    dxLeftCol = 0 & dyUpCol = 0
    dxRightCol = 0 & dyDownCol = 0
    xCol = 0 & yCol = 0 & qCol = 0

    text = ''
    title = '' & subTitle = ''
    xMin = 0 & xMax = 0 & xStep = 1 & xTitle = ''
    yMin = 0 & yMax = 0 & yStep = 1 & yTitle = ''
    noXMax = 0 & noYMax = 0
    noXMin = 0 & noYMin = 0
    xValues = 0
    color = 255B

; decoding header:

    headerLine = ''
    line = 1 & keyCol = 0
    ReadF,unit,headerLine 
    headerLine = StrTrim(headerLine,2)
    firstChar = StrMid( headerLine,0,1)

    WHILE (NOT IsDigit(firstChar))  DO BEGIN

      IF  (firstChar NE '%') AND (firstChar NE '') THEN BEGIN
        equalSignPos = StrPos( headerLine, '=' )
        IF equalSignPos EQ -1 THEN BEGIN
          Error, 9, 'MDisp Read'
          Free_LUn,unit
          RETURN
        ENDIF
        keyword = StrTrim( StrMid( headerLine, 0, equalSignPos ),2 )
        keyword = StrUpcase( keyword )
        value = StrTrim( StrMid(headerLine, equalSignPos+1, $
	StrLen(headerLine)-1),2)
        CASE keyword OF
          'AUTOR': text = text + value
          'COLOR': color = value
          'DXLEFT': dxLeft = FLOAT(value)
          'DXRIGHT': dxRight = FLOAT(value)
          'DXLEFTCOL': BEGIN 
		dxLeftCol=FIX(value) & keyCol=keyCol+1
	               END
          'DXRIGHTCOL': BEGIN
                                        dxRightCol = FIX(value) & keyCol = keyCol+1
                                     END
          'DYUP': dyUp = FLOAT(value)
          'DYDOWN': dyDown = FLOAT(value)
          'DYUPCOL': BEGIN 
	             dyUpCol=FIX(value) & keyCol=keyCol+1
                              END
          'DYDOWNCOL': BEGIN
                                       dyDownCol = FIX(value) & keyCol = keyCol+1
         	                  END
          'INSTRUMENT': text = text + value
          'LINESTYLE': lineStyle = 0
          'QCOL': BEGIN qCol = Fix(value) & keyCol=keyCol+1 & END
          'QUALIFIER': q = FIX(value)
          'SOURCE' : text = text + value
          'SUBTITLE': subtitle = value
          'THICK': thick = value
          'TITLE': title = value
          'XCOL': BEGIN 
	        xCol = FIX(value) & keyCol =keyCol+1 
	      END
          'XMIN': BEGIN
                        xMin = FLOAT(value)
                        noXMax = 1
                      END
           'XMAX': BEGIN
                           xMax = FLOAT(value)
                           noXMin = 1
                         END
          'XSTEP': xStep = FLOAT(value)
          'XTITLE': xTitle = value
          'XVALUES': xValues = 1
          'YCOL': BEGIN  
                            yCol = FIX(value) & keyCol =keyCol+1
                          END
          'YMIN': BEGIN
                        yMin = FLOAT(value)
                        noYMax = 1
                      END
          'YMAX': BEGIN
                           yMax = FLOAT(value)
                           noYMin = 1
                         END
          'YSTEP': yStep = FLOAT(value)
          'YTITLE': ytitle = value
          ELSE: BEGIN
            Error, 10, 'Mdisp Read'
            Free_LUn,unit
            RETURN
          END
        ENDCASE
      ENDIF
      line = line+1
      ReadF,unit,headerLine
      headerLine = StrTrim( headerLine, 2)
      firstChar = StrMid( headerLine, 0, 1)
    ENDWHILE

    line = line-1
    Free_Lun, unit

    IF keyCol EQ 0 THEN BEGIN 
; determine nb of columns (default values)
      headerLine = headerLine + ' '
      nbOfCols = 0
      valLength = 0
        WHILE (headerLine  NE '') AND (valLength NE -1) DO BEGIN
          valLength = StrPos( headerLine,' ',0 ) 
          status = Execute(varList(nbOfCols) + $
	'Col = ' + StrTrim(nbOfCols+1) )
          nbOfCols = nbOfCols+1
          headerLine = StrMid( headerLine, valLength, StrLen(headerLine))
          headerLine = StrTrim(headerLine,1)
        ENDWHILE
        keyCol = nbOfCols
    ENDIF ELSE nbOfCols = keyCol ; cols determined with keywords.

    IF nbOfCols EQ 1 THEN BEGIN
      parList = 'y,'
      parColList = 'yCol'
      yCol = 1
    ENDIF ELSE BEGIN
      parList = '' & parColList = ''
      FOR i = 1, 7 DO BEGIN
        status = Execute( 'test = ' + varList(i-1)+'Col NE 0 ')
        IF test NE 0 THEN BEGIN
          parList = parList + varList(i-1) + ', '
          IF i EQ 1 THEN parColList =  varList(i-1) + 'Col'  $
          ELSE parColList = parColList  +  ',' + varList(i-1) + 'Col' 
        ENDIF
      ENDFOR
    ENDELSE
  
    varPlace = [ xCol, yCol, qCol, dyUpCol, dxLeftCol, dyDownCol, $
	 dxRightCol ]

  ENDIF ELSE  BEGIN; mdisp keywords setting finished.
    keyCol = 1
    nbOfCols = N_Elements( Where( varPlace NE 0 ) )
  ENDELSE

  Print,' Reading data from file  ', file, ' ... '

  Read_Array, file, tempArr, SKIP = line

  FOR i=0,6 DO BEGIN
    IF varPlace(i) NE 0 THEN $
      execStatus = Execute( varList(i)+'= Transpose( tempArr( '+ $
	StrTrim( varPlace(i)-1 )+ ', *))' )
  ENDFOR


  IF N_Elements( dyUp ) NE 0 THEN BEGIN
    dy = dyUp
    IF N_Elements( dyDown ) NE 0 THEN dy = [ [dy], [dyDown] ]
  ENDIF
  IF N_Elements( dxLeft ) NE 0 THEN BEGIN
    dx = dxLeft
    IF N_Elements( dxRight ) NE 0 THEN dx = [ [dx], [dxRight] ]
  ENDIF

  n = N_Elements( y )

  IF N_Elements( q ) NE 0 THEN BEGIN
    IF nbOfCols GE 3 THEN QualifTransform, q, lineStyle, userSym $
    ELSE  IF N_Elements(q) EQ 1 THEN BEGIN
      QualifTransform, q, lineStyle, userSym
      q = Replicate( q, n )
      IF q(0) EQ 8 THEN userSym = Replicate( userSym, n )
    ENDIF
  ENDIF

  IF noXMin NE 0 OR noXMax NE 0  THEN BEGIN
    IF N_Elements( xStep ) EQ 0 THEN xStep = 1
    IF noXMax THEN x = DIndGen( n )*xStep + xMin $
    ELSE x =  xMax - DIndGen(n)*xStep 
  ENDIF
    
  status = "Done"
  RETURN

Exit:
  Message, !err_string, /INFO
  status = "Not Done"

END


