;+
; NAME:
;	SCA_Read
; PURPOSE:
;	Reads SCA files.
; CATEGORY:
;	Input/Output
; CALLING SEQUENCES:
;	SCA_Read, file, image, [ xAxis, yAxis ]
;	SCA_Read, file, image,  nx, ny, x1, y1, dx, dy
; INPUT ARGUMENTS:
;	file: complete name of the file to be read
; OUTPUT ARGUMENTS:
;	image: the two-dimensionnal array
;	xAxis, yAxis: the axes of "image"
;	nx, ny: the array dimensions
;	dx, dy: the image steps between two pixels
;	x1, y1: the first value of the axes.
; KEYWORDS:
;	TITLE: the title of the image
;	XLAB, YLAB, ZLAB: the titles of the axes involved
;	NAME: the name of the image
;	VMS: if present, the file is assumed to have the
;		"variable record length" from the VMS
;		fortran write. The file will be read, and 
;		directly rewritten in Unix SCA format.
;	TRANSFORM: transforms microseconds in seconds
;		in case of RAG data.
; MODIFICATION HISTORY:
;	Created, 4/91, A.Csillaghy, ETHZ
;	Modified, Jan.93, A.Cs
;-

PRO Sca_Read_VMS, unit, image, nx, ny, x1, y1, dx, dy, header

  Print, 'Reading file records ... '
  Read_Data_Byte, unit, image

  Print, 'Decoding header and constants ... '
  header = String( image(0:148) )

  nx = Long( image(IndGen(4)+149), 0 )
  ny = Long( image(IndGen(4)+153), 0 )
  x1 = VMSFloat( image(IndGen(4) + 157 ))
  y1 = VMSFloat( image(IndGen(4) + 161 ))
  dx = VMSFloat( image(IndGen(4) + 165 ))
  dy  = VMSFloat( image(IndGen(4) + 169 ))

  IF nx EQ 0 THEN nx = 1
  IF ny EQ 0 THEN ny = 1
  IF dx EQ 0. THEN dx = 1.
  IF dy EQ 0. THEN dy = 1.

  Print, 'Decoding image ... '
  imTmp = VMSFloat( image(173:N_Elements(image)-1) )
  image = FltArr(nx, ny,/NOZERO)
  image(0) = imTmp

END ; SCA_Read_VMS


PRO Sca_Read_Unix, unit, image, nx, ny, x1, y1, dx, dy, header, $
	transform

  Print,' Reading header ... '
  header = STRING(REPLICATE(32B,149))
  ReadU,unit,header

  Print,' Reading constants ... '
  nx=0L 	& ny=0L
  x1=0.   & y1=0.  & dx=0.  & dy=0.
  ReadU,unit,nx,ny,x1,y1,dx,dy

  IF dx EQ 0 THEN dx = 1.
  IF dy EQ 0 THEN dy = 1.

  IF dx GT 1e4 THEN BEGIN
    IF transform EQ 1 THEN dx = dx/1e6 $
    ELSE BEGIN
      rep = ''
      Print, 'The x step is probably in microseconds.'
      Read, 'Do you want to transform it in seconds [y] ? ',rep
      IF StrUpcase(rep) NE 'N' THEN dx = dx / 1e6
    ENDELSE
  ENDIF

  Print,' Reading image array ... '
  image = FltArr(nx,ny)
  ReadU,unit,image

END ; SCA_Read_Unix

PRO SCA_Read,file,image,xAxis, yAxis, x1, y1, dx, dy, $
	TITLE = title, NAME = name, VMS = vms, $
	XLAB = xLab, YLAB = yLab, ZLAB = zLab, $
	TRANSFORM = transform


  ON_IOError, ErrLabel

  vmsFile = Keyword_Set(VMS)
  IF Keyword_Set( TRANSFORM ) THEN transform = 1 $
  ELSE transform = 0
  nbParams = N_Params()
  IF nbParams LT 1 THEN BEGIN
    Message, 'Wrong number of parameters ... ', /INFO, /CONT
    Print, '[ SCA_Read,file,image,xAxis, yAxis ]'
    RETURN
  ENDIF

  fileToR=StrTrim(file,2)
  IF vmsFile THEN OpenR, unit, fileToR, /GET_LUN $
  ELSE OpenR, unit, fileToR, /GET_LUN, /F77_UNFORMATTED 

  Print, 'Sca File Read'

  IF vmsFile THEN SCA_Read_VMS, unit, image, nx, ny, x1, y1, $
	dx, dy, header $
  ELSE SCA_Read_Unix, unit, image, nx, ny, x1, y1, $
	dx, dy, header, transform

  Free_LUn,unit

  title = STRTRIM(STRMID(header(0),0,80),2)
  xLab = STRTRIM(STRMID(header(0),80,20),2)
  yLab = STRTRIM(STRMID(header(0),100,20),2) 
  zLab = STRTRIM(STRMID(header(0),120,20),2) 

  Print,' Header: '
  Print,'        title:   '+title
  Print,'        x label: '+xLab
  Print,'        y label: '+yLab
  Print,'        z label: '+zLab
  Print,' Constants:'
  Print,'           x-dimension: '+String(nx)
  Print,'           y-dimension: '+String(ny)
  Print,'           x-step: '+String(dx)
  Print,'           y-step: '+String(dy)
  Print,'           x first value: '+String(x1)
  Print,'           y first value: '+String(y1)

  minZ=StrTrim(MIN(image) ,2) 
  maxZ=StrTrim(MAX(image),2)
  Print,' The image range is: ['+minZ+'..'+maxZ+']'

  IF vmsFile THEN $
	Sca_Write, image, x1, y1, dx, dy, $
		FILENAME = fileToR, TITLE = title, $
		NAME = name, XLAB = xLab, $
		YLAB = yLab, ZLAB = zLab

  IF (nbParams GE 3) AND (nbParams LE 4) THEN BEGIN
    xAxis = FIndGen( nx )* dx + x1
    yAxis = FIndGen( ny )* dy + y1
  ENDIF ELSE IF (nbParams GT 4) THEN BEGIN
    xAxis = nx
    yAxis = ny
  ENDIF

  RETURN

  ErrLabel:
    Print, 'I/O Error in the sca read procedure '
    Print, !err_string
  
END
